@@ -2,11 +2,13 @@
 '''
 Support for the Amazon Simple Queue Service.
 '''
+from __future__ import absolute_import
 import logging
 import json
 
 # Import salt libs
 import salt.utils
+import salt.ext.six as six
 
 log = logging.getLogger(__name__)
 
@@ -17,7 +19,7 @@ def __virtual__():
     if salt.utils.which('aws'):
         # awscli is installed, load the module
         return True
-    return False
+    return (False, 'The module aws_sqs could not be loaded: aws command not found')
 
 
 def _region(region):
@@ -51,7 +53,7 @@ def _run_aws(cmd, region, opts, user, **kwargs):
         kwargs['max-number-of-messages'] = num
 
     _formatted_args = [
-        '--{0} "{1}"'.format(k, v) for k, v in kwargs.iteritems()]
+        '--{0} "{1}"'.format(k, v) for k, v in six.iteritems(kwargs)]
 
     cmd = 'aws sqs {cmd} {args} {region} {out}'.format(
         cmd=cmd,
@@ -219,7 +221,7 @@ def delete_queue(name, region, opts=None, user=None):
     url_map = _parse_queue_list(queues)
 
     logger = logging.getLogger(__name__)
-    logger.debug('map ' + unicode(url_map))
+    logger.debug('map ' + six.text_type(url_map))
     if name in url_map:
         delete = {'queue-url': url_map[name]}
 
