@@ -5,7 +5,7 @@ Connection module for Amazon VPC
 .. versionadded:: 2014.7.0
 
 :configuration: This module accepts explicit VPC credentials but can also
-    utilize IAM roles assigned to the instance trough Instance Profiles.
+    utilize IAM roles assigned to the instance through Instance Profiles.
     Dynamic credentials are then automatically obtained from AWS API and no
     further configuration is necessary. More Information available at:
 
@@ -117,26 +117,35 @@ def __virtual__():
     # which was added in boto 2.8.0
     # https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
     if not HAS_BOTO:
-        return False
+        return (False, 'The boto_vpc module could not be loaded: boto libraries not found')
     elif _LooseVersion(boto.__version__) < _LooseVersion(required_boto_version):
-        return False
+        return (False, 'The boto_vpc module could not be loaded: boto library is not required version 2.8.0')
     else:
 
         return True
 
 
-def __init__(opts, pack=None):
+def __init__(opts):
     salt.utils.compat.pack_dunder(__name__)
     if HAS_BOTO:
-        __utils__['boto.assign_funcs'](__name__, 'vpc', pack=pack)
+        __utils__['boto.assign_funcs'](__name__, 'vpc', pack=__salt__)
 
 
-def _check_vpc(vpc_id, vpc_name, region, key, keyid, profile):
+def check_vpc(vpc_id=None, vpc_name=None, region=None, key=None,
+              keyid=None, profile=None):
     '''
     Check whether a VPC with the given name or id exists.
     Returns the vpc_id or None. Raises SaltInvocationError if
     both vpc_id and vpc_name are None. Optionally raise a
     CommandExecutionError if the VPC does not exist.
+
+    .. versionadded:: 2016.3.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion boto_vpc.check_vpc vpc_name=myvpc profile=awsprofile
     '''
 
     if not _exactly_one((vpc_name, vpc_id)):
@@ -633,7 +642,7 @@ def describe(vpc_id=None, vpc_name=None, region=None, key=None,
 
     try:
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
-        vpc_id = _check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
+        vpc_id = check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
         if not vpc_id:
             return {'vpc': None}
 
@@ -768,16 +777,16 @@ def create_subnet(vpc_id=None, cidr_block=None, vpc_name=None,
     '''
 
     try:
-        vpc_id = _check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
+        vpc_id = check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
         if not vpc_id:
             return {'created': False, 'error': {'message': 'VPC {0} does not exist.'.format(vpc_name or vpc_id)}}
     except BotoServerError as e:
         return {'created': False, 'error': salt.utils.boto.get_error(e)}
 
-    return _create_resource('subnet', name=subnet_name, tags=tags,
-                            vpc_id=vpc_id, cidr_block=cidr_block,
-                            region=region, key=key, keyid=keyid,
-                            profile=profile, availability_zone=availability_zone)
+    return _create_resource('subnet', name=subnet_name, tags=tags, vpc_id=vpc_id,
+                            availability_zone=availability_zone,
+                            cidr_block=cidr_block, region=region, key=key,
+                            keyid=keyid, profile=profile)
 
 
 def delete_subnet(subnet_id=None, subnet_name=None, region=None, key=None,
@@ -1027,7 +1036,7 @@ def create_internet_gateway(internet_gateway_name=None, vpc_id=None,
 
     try:
         if vpc_id or vpc_name:
-            vpc_id = _check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
+            vpc_id = check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
             if not vpc_id:
                 return {'created': False,
                         'error': {'message': 'VPC {0} does not exist.'.format(vpc_name or vpc_id)}}
@@ -1197,7 +1206,7 @@ def create_dhcp_options(domain_name=None, domain_name_servers=None, ntp_servers=
 
     try:
         if vpc_id or vpc_name:
-            vpc_id = _check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
+            vpc_id = check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
             if not vpc_id:
                 return {'created': False,
                         'error': {'message': 'VPC {0} does not exist.'.format(vpc_name or vpc_id)}}
@@ -1218,6 +1227,45 @@ def create_dhcp_options(domain_name=None, domain_name_servers=None, ntp_servers=
         return {'created': False, 'error': salt.utils.boto.get_error(e)}
 
 
+def get_dhcp_options(dhcp_options_name=None, dhcp_options_id=None,
+                     region=None, key=None, keyid=None, profile=None):
+    '''
+    Return a dict with the current values of the requested DHCP options set
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion boto_vpc.get_dhcp_options 'myfunnydhcpoptionsname'
+
+    .. versionadded:: 2016.3.0
+    '''
+    if not any((dhcp_options_name, dhcp_options_id)):
+        raise SaltInvocationError('At least one of the following must be specified: '
+                                  'dhcp_options_name, dhcp_options_id.')
+
+    if not dhcp_options_id and dhcp_options_name:
+        dhcp_options_id = _get_resource_id('dhcp_options', dhcp_options_name,
+                                            region=region, key=key,
+                                            keyid=keyid, profile=profile)
+    if not dhcp_options_id:
+        return {'dhcp_options': {}}
+
+    try:
+        conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+        r = conn.get_all_dhcp_options(dhcp_options_ids=[dhcp_options_id])
+    except BotoServerError as e:
+        return {'error': salt.utils.boto.get_error(e)}
+
+    if not r:
+        return {'dhcp_options': None}
+
+    keys = ('domain_name', 'domain_name_servers', 'ntp_servers',
+            'netbios_name_servers', 'netbios_node_type')
+
+    return {'dhcp_options': dict((k, r[0].options.get(k)) for k in keys)}
+
+
 def delete_dhcp_options(dhcp_options_id=None, dhcp_options_name=None,
                         region=None, key=None, keyid=None, profile=None):
     '''
@@ -1255,7 +1303,7 @@ def associate_dhcp_options_to_vpc(dhcp_options_id, vpc_id=None, vpc_name=None,
 
     '''
     try:
-        vpc_id = _check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
+        vpc_id = check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
         if not vpc_id:
             return {'associated': False,
                     'error': {'message': 'VPC {0} does not exist.'.format(vpc_name or vpc_id)}}
@@ -1275,12 +1323,12 @@ def associate_new_dhcp_options_to_vpc(vpc_id, domain_name=None, domain_name_serv
                                       netbios_name_servers=None, netbios_node_type=None,
                                       region=None, key=None, keyid=None, profile=None):
     '''
-    ..deprecated:: Boron
+    ..deprecated:: Carbon
         This function has been deprecated in favor of
         :py:func:`boto_vpc.create_dhcp_options <salt.modules.boto_vpc.create_dhcp_options>`,
         which now takes vpc_id or vpc_name as kwargs.
 
-        This function will be removed in the Salt Boron release.
+        This function will be removed in the Salt Carbon release.
 
     Given valid DHCP options and a valid VPC id, create and associate the DHCP options record with the VPC.
 
@@ -1292,9 +1340,9 @@ def associate_new_dhcp_options_to_vpc(vpc_id, domain_name=None, domain_name_serv
 
     '''
     salt.utils.warn_until(
-        'Boron',
+        'Carbon',
         'Support for \'associate_new_dhcp_options_to_vpc\' has been deprecated '
-        'and will be removed in Salt Boron. Please use \'create_dhcp_options\' instead.'
+        'and will be removed in Salt Carbon. Please use \'create_dhcp_options\' instead.'
     )
 
     return create_dhcp_options(vpc_id=vpc_id, domain_name=domain_name,
@@ -1353,7 +1401,7 @@ def create_network_acl(vpc_id=None, vpc_name=None, network_acl_name=None,
     _id = vpc_name or vpc_id
 
     try:
-        vpc_id = _check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
+        vpc_id = check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
     except BotoServerError as e:
         return {'created': False, 'error': salt.utils.boto.get_error(e)}
 
@@ -1505,12 +1553,12 @@ def associate_network_acl_to_subnet(network_acl_id=None, subnet_id=None,
 def associate_new_network_acl_to_subnet(vpc_id, subnet_id, network_acl_name=None, tags=None,
                                         region=None, key=None, keyid=None, profile=None):
     '''
-    ..deprecated:: Boron
+    ..deprecated:: Carbon
         This function has been deprecated in favor of
         :py:func:`boto_vpc.create_network_acl <salt.modules.boto_vpc.create_network_acl>`,
         which now takes subnet_id or subnet_name as kwargs.
 
-        This function will be removed in the Salt Boron release.
+        This function will be removed in the Salt Carbon release.
 
     Given a vpc ID and a subnet ID, associates a new network act to a subnet.
 
@@ -1524,9 +1572,9 @@ def associate_new_network_acl_to_subnet(vpc_id, subnet_id, network_acl_name=None
         salt myminion boto_vpc.associate_new_network_acl_to_subnet 'vpc-6b1fe402' 'subnet-6a1fe403'
     '''
     salt.utils.warn_until(
-        'Boron',
+        'Carbon',
         'Support for \'associate_new_network_acl_to_subnet\' has been deprecated '
-        'and will be removed in Salt Boron. Please use \'create_network_acl\' instead.'
+        'and will be removed in Salt Carbon. Please use \'create_network_acl\' instead.'
     )
 
     return create_network_acl(vpc_id=vpc_id, subnet_id=subnet_id,
@@ -1564,7 +1612,7 @@ def disassociate_network_acl(subnet_id=None, vpc_id=None, subnet_name=None, vpc_
                         'error': {'message': 'Subnet {0} does not exist.'.format(subnet_name)}}
 
         if vpc_name or vpc_id:
-            vpc_id = _check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
+            vpc_id = check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
 
         conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
         association_id = conn.disassociate_network_acl(subnet_id, vpc_id=vpc_id)
@@ -1726,7 +1774,7 @@ def create_route_table(vpc_id=None, vpc_name=None, route_table_name=None,
         salt myminion boto_vpc.create_route_table vpc_name='myvpc' \\
                 route_table_name='myroutetable'
     '''
-    vpc_id = _check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
+    vpc_id = check_vpc(vpc_id, vpc_name, region, key, keyid, profile)
     if not vpc_id:
         return {'created': False, 'error': {'message': 'VPC {0} does not exist.'.format(vpc_name or vpc_id)}}
 
