@@ -4,7 +4,9 @@ Module for handling openstack keystone calls.
 
 :optdepends:    - keystoneclient Python adapter
 :configuration: This module is not usable until the following are specified
-    either in a pillar or in the minion's config file::
+    either in a pillar or in the minion's config file:
+
+    .. code-block:: yaml
 
         keystone.user: admin
         keystone.password: verybadpass
@@ -12,14 +14,17 @@ Module for handling openstack keystone calls.
         keystone.tenant_id: f80919baedab48ec8931f200c65a50df
         keystone.auth_url: 'http://127.0.0.1:5000/v2.0/'
 
-        OR (for token based authentication)
+    OR (for token based authentication)
+
+    .. code-block:: yaml
 
         keystone.token: 'ADMIN'
         keystone.endpoint: 'http://127.0.0.1:35357/v2.0'
 
     If configuration for multiple openstack accounts is required, they can be
-    set up as different configuration profiles:
-    For example::
+    set up as different configuration profiles. For example:
+
+    .. code-block:: yaml
 
         openstack1:
           keystone.user: admin
@@ -37,20 +42,33 @@ Module for handling openstack keystone calls.
 
     With this configuration in place, any of the keystone functions can make use
     of a configuration profile by declaring it explicitly.
-    For example::
+    For example:
+
+    .. code-block:: bash
 
         salt '*' keystone.tenant_list profile=openstack1
 '''
 
+# Import Python libs
+from __future__ import absolute_import
+import logging
+
+# Import Salt Libs
+import salt.ext.six as six
+
 # Import third party libs
 HAS_KEYSTONE = False
 try:
+    # pylint: disable=import-error
     from keystoneclient.v2_0 import client
     import keystoneclient.exceptions
+    # pylint: enable=import-error
     HAS_KEYSTONE = True
 except ImportError:
     pass
 
+log = logging.getLogger(__name__)
+
 
 def __virtual__():
     '''
@@ -59,16 +77,20 @@ def __virtual__():
     '''
     if HAS_KEYSTONE:
         return 'keystone'
-    return False
+    return (False, 'keystone execution module cannot be loaded: keystoneclient python library not available.')
 
 __opts__ = {}
 
 
 def auth(profile=None, **connection_args):
     '''
-    Set up keystone credentials
+    Set up keystone credentials. Only intended to be used within Keystone-enabled modules.
 
-    Only intended to be used within Keystone-enabled modules
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' keystone.auth
     '''
 
     if profile:
@@ -119,8 +141,8 @@ def ec2_credentials_create(user_id=None, name=None,
 
         salt '*' keystone.ec2_credentials_create name=admin tenant=admin
         salt '*' keystone.ec2_credentials_create \
-            user_id=c965f79c4f864eaaa9c3b41904e67082 \
-            tenant_id=722787eb540849158668370dc627ec5f
+user_id=c965f79c4f864eaaa9c3b41904e67082 \
+tenant_id=722787eb540849158668370dc627ec5f
     '''
     kstone = auth(profile, **connection_args)
 
@@ -153,10 +175,9 @@ def ec2_credentials_delete(user_id=None, name=None, access_key=None,
     .. code-block:: bash
 
         salt '*' keystone.ec2_credentials_delete \
-            860f8c2c38ca4fab989f9bc56a061a64
-            access_key=5f66d2f24f604b8bb9cd28886106f442
+860f8c2c38ca4fab989f9bc56a061a64 access_key=5f66d2f24f604b8bb9cd28886106f442
         salt '*' keystone.ec2_credentials_delete name=admin \
-            access_key=5f66d2f24f604b8bb9cd28886106f442
+access_key=5f66d2f24f604b8bb9cd28886106f442
     '''
     kstone = auth(profile, **connection_args)
 
@@ -289,7 +310,8 @@ def endpoint_create(service, publicurl=None, internalurl=None, adminurl=None,
             'http://internal/url' 'http://adminurl/url' region
     '''
     kstone = auth(profile, **connection_args)
-    keystone_service = service_get(name=service, **connection_args)
+    keystone_service = service_get(name=service, profile=profile,
+                                   **connection_args)
     if not keystone_service or 'Error' in keystone_service:
         return {'Error': 'Could not find the specified service'}
     kstone.endpoints.create(region=region,
@@ -297,7 +319,7 @@ def endpoint_create(service, publicurl=None, internalurl=None, adminurl=None,
                             publicurl=publicurl,
                             adminurl=adminurl,
                             internalurl=internalurl)
-    return endpoint_get(service, **connection_args)
+    return endpoint_get(service, profile, **connection_args)
 
 
 def endpoint_delete(service, profile=None, **connection_args):
@@ -322,7 +344,9 @@ def endpoint_delete(service, profile=None, **connection_args):
 
 def role_create(name, profile=None, **connection_args):
     '''
-    Create named role
+    Create a named role.
+
+    CLI Example:
 
     .. code-block:: bash
 
@@ -421,7 +445,7 @@ def service_create(name, service_type, description=None, profile=None,
     .. code-block:: bash
 
         salt '*' keystone.service_create nova compute \
-                'OpenStack Compute Service'
+'OpenStack Compute Service'
     '''
     kstone = auth(profile, **connection_args)
     service = kstone.services.create(name, service_type, description)
@@ -686,7 +710,13 @@ def user_get(user_id=None, name=None, profile=None, **connection_args):
                 break
     if not user_id:
         return {'Error': 'Unable to resolve user id'}
-    user = kstone.users.get(user_id)
+    try:
+        user = kstone.users.get(user_id)
+    except keystoneclient.exceptions.NotFound:
+        msg = 'Could not find user \'{0}\''.format(user_id)
+        log.error(msg)
+        return {'Error': msg}
+
     ret[user.name] = {'id': user.id,
                       'name': user.name,
                       'email': user.email,
@@ -777,9 +807,9 @@ def user_update(user_id=None, name=None, email=None, enabled=None,
         enabled = user.enabled
     kstone.users.update(user=user_id, name=name, email=email, enabled=enabled)
     if tenant:
-        for t in kstone.tenants.list():
-            if t.name == tenant:
-                tenant_id = t.id
+        for tnt in kstone.tenants.list():
+            if tnt.name == tenant:
+                tenant_id = tnt.id
                 break
         kstone.users.update_tenant(user_id, tenant_id)
     ret = 'Info updated for user ID {0}'.format(user_id)
@@ -817,7 +847,8 @@ def user_verify_password(user_id=None, name=None, password=None,
               'auth_url': auth_url}
     try:
         userauth = client.Client(**kwargs)
-    except keystoneclient.exceptions.Unauthorized:
+    except (keystoneclient.exceptions.Unauthorized,
+            keystoneclient.exceptions.AuthorizationFailure):
         return False
     return True
 
@@ -861,9 +892,9 @@ def user_role_add(user_id=None, user=None, tenant_id=None,
     .. code-block:: bash
 
         salt '*' keystone.user_role_add \
-            user_id=298ce377245c4ec9b70e1c639c89e654 \
-            tenant_id=7167a092ece84bae8cead4bf9d15bb3b \
-            role_id=ce377245c4ec9b70e1c639c89e8cead4
+user_id=298ce377245c4ec9b70e1c639c89e654 \
+tenant_id=7167a092ece84bae8cead4bf9d15bb3b \
+role_id=ce377245c4ec9b70e1c639c89e8cead4
         salt '*' keystone.user_role_add user=admin tenant=admin role=admin
     '''
     kstone = auth(profile, **connection_args)
@@ -871,8 +902,8 @@ def user_role_add(user_id=None, user=None, tenant_id=None,
         user_id = user_get(name=user, profile=profile,
                            **connection_args)[user]['id']
     else:
-        user = user_get(user_id, profile=profile,
-                        **connection_args).iterkeys().next()['name']
+        user = next(six.iterkeys(user_get(user_id, profile=profile,
+                                          **connection_args)))['name']
     if not user_id:
         return {'Error': 'Unable to resolve user id'}
 
@@ -880,8 +911,8 @@ def user_role_add(user_id=None, user=None, tenant_id=None,
         tenant_id = tenant_get(name=tenant, profile=profile,
                                **connection_args)[tenant]['id']
     else:
-        tenant = tenant_get(tenant_id, profile=profile,
-                            **connection_args).iterkeys().next()['name']
+        tenant = next(six.iterkeys(tenant_get(tenant_id, profile=profile,
+                                              **connection_args)))['name']
     if not tenant_id:
         return {'Error': 'Unable to resolve tenant id'}
 
@@ -889,8 +920,8 @@ def user_role_add(user_id=None, user=None, tenant_id=None,
         role_id = role_get(name=role, profile=profile,
                            **connection_args)[role]['id']
     else:
-        role = role_get(role_id, profile=profile,
-                        **connection_args).iterkeys().next()['name']
+        role = next(six.iterkeys(role_get(role_id, profile=profile,
+                                          **connection_args)))['name']
     if not role_id:
         return {'Error': 'Unable to resolve role id'}
 
@@ -910,9 +941,9 @@ def user_role_remove(user_id=None, user=None, tenant_id=None,
     .. code-block:: bash
 
         salt '*' keystone.user_role_remove \
-            user_id=298ce377245c4ec9b70e1c639c89e654 \
-            tenant_id=7167a092ece84bae8cead4bf9d15bb3b \
-            role_id=ce377245c4ec9b70e1c639c89e8cead4
+user_id=298ce377245c4ec9b70e1c639c89e654 \
+tenant_id=7167a092ece84bae8cead4bf9d15bb3b \
+role_id=ce377245c4ec9b70e1c639c89e8cead4
         salt '*' keystone.user_role_remove user=admin tenant=admin role=admin
     '''
     kstone = auth(profile, **connection_args)
@@ -920,8 +951,8 @@ def user_role_remove(user_id=None, user=None, tenant_id=None,
         user_id = user_get(name=user, profile=profile,
                            **connection_args)[user]['id']
     else:
-        user = user_get(user_id, profile=profile,
-                        **connection_args).iterkeys().next()['name']
+        user = next(six.iterkeys(user_get(user_id, profile=profile,
+                                          **connection_args)))['name']
     if not user_id:
         return {'Error': 'Unable to resolve user id'}
 
@@ -929,8 +960,8 @@ def user_role_remove(user_id=None, user=None, tenant_id=None,
         tenant_id = tenant_get(name=tenant, profile=profile,
                                **connection_args)[tenant]['id']
     else:
-        tenant = tenant_get(tenant_id, profile=profile,
-                            **connection_args).iterkeys().next()['name']
+        tenant = next(six.iterkeys(tenant_get(tenant_id, profile=profile,
+                                              **connection_args)))['name']
     if not tenant_id:
         return {'Error': 'Unable to resolve tenant id'}
 
@@ -938,7 +969,7 @@ def user_role_remove(user_id=None, user=None, tenant_id=None,
         role_id = role_get(name=role, profile=profile,
                            **connection_args)[role]['id']
     else:
-        role = role_get(role_id).iterkeys().next()['name']
+        role = next(six.iterkeys(role_get(role_id)))['name']
     if not role_id:
         return {'Error': 'Unable to resolve role id'}
 
@@ -957,8 +988,8 @@ def user_role_list(user_id=None, tenant_id=None, user_name=None,
     .. code-block:: bash
 
         salt '*' keystone.user_role_list \
-            user_id=298ce377245c4ec9b70e1c639c89e654 \
-            tenant_id=7167a092ece84bae8cead4bf9d15bb3b
+user_id=298ce377245c4ec9b70e1c639c89e654 \
+tenant_id=7167a092ece84bae8cead4bf9d15bb3b
         salt '*' keystone.user_role_list user_name=admin tenant_name=admin
     '''
     kstone = auth(profile, **connection_args)
@@ -1004,12 +1035,12 @@ def _item_list(profile=None, **connection_args):
         #        }
     return ret
 
-    #The following is a list of functions that need to be incorporated in the
-    #keystone module. This list should be updated as functions are added.
+    # The following is a list of functions that need to be incorporated in the
+    # keystone module. This list should be updated as functions are added.
     #
-    #endpoint-create     Create a new endpoint associated with a service
-    #endpoint-delete     Delete a service endpoint
-    #discover            Discover Keystone servers and show authentication
-    #                    protocols and
-    #bootstrap           Grants a new role to a new user on a new tenant, after
-    #                    creating each.
+    # endpoint-create     Create a new endpoint associated with a service
+    # endpoint-delete     Delete a service endpoint
+    # discover            Discover Keystone servers and show authentication
+    #                     protocols and
+    # bootstrap           Grants a new role to a new user on a new tenant, after
+    #                     creating each.
