@@ -4,6 +4,7 @@ Configure ``portage(5)``
 '''
 
 # Import python libs
+from __future__ import absolute_import
 import os
 import shutil
 
@@ -11,6 +12,8 @@ import shutil
 import salt.utils
 
 # Import third party libs
+import salt.ext.six as six
+# pylint: disable=import-error
 try:
     import portage
     HAS_PORTAGE = True
@@ -26,6 +29,7 @@ except ImportError:
             HAS_PORTAGE = True
         except ImportError:
             pass
+# pylint: enable=import-error
 
 
 BASE_PATH = '/etc/portage/package.{0}'
@@ -39,17 +43,51 @@ def __virtual__():
     '''
     if HAS_PORTAGE and __grains__['os'] == 'Gentoo':
         return 'portage_config'
-    return False
+    return (False, 'portage_config execution module cannot be loaded: only available on Gentoo with portage installed.')
+
+
+def _get_portage():
+    '''
+    portage module must be reloaded or it can't catch the changes
+    in portage.* which had been added after when the module was loaded
+    '''
+    return reload(portage)
 
 
 def _porttree():
     return portage.db[portage.root]['porttree']
 
 
+def _get_config_file(conf, atom):
+    '''
+    Parse the given atom, allowing access to its parts
+    Success does not mean that the atom exists, just that it
+    is in the correct format.
+    Returns none if the atom is invalid.
+    '''
+    if '*' in atom:
+        parts = portage.dep.Atom(atom, allow_wildcard=True)
+        if not parts:
+            return
+        if parts.cp == '*/*':
+            # parts.repo will be empty if there is no repo part
+            relative_path = parts.repo or "gentoo"
+        else:
+            relative_path = os.path.join(*[x for x in os.path.split(parts.cp) if x != '*'])
+    else:
+        relative_path = _p_to_cp(atom)
+        if not relative_path:
+            return
+
+    complete_file_path = BASE_PATH.format(conf) + '/' + relative_path
+
+    return complete_file_path
+
+
 def _p_to_cp(p):
     '''
     Convert a package name or a DEPEND atom to category/package format.
-    Raises an exception if program name is ambigous.
+    Raises an exception if program name is ambiguous.
     '''
     ret = _porttree().dbapi.xmatch("match-all", p)
     if ret:
@@ -57,11 +95,29 @@ def _p_to_cp(p):
     return None
 
 
+def _get_cpv(cp, installed=True):
+    '''
+    add version to category/package
+    @cp - name of package in format category/name
+    @installed - boolean value, if False, function returns cpv
+    for latest available package
+    '''
+    if installed:
+        return _get_portage().db[portage.root]['vartree'].dep_bestmatch(cp)
+    else:
+        return _porttree().dep_bestmatch(cp)
+
+
 def enforce_nice_config():
     '''
     Enforce a nice tree structure for /etc/portage/package.* configuration
     files.
 
+    .. seealso::
+       :py:func:`salt.modules.ebuild.ex_mod_init`
+         for information on automatically running this when pkg is used.
+
+
     CLI Example:
 
     .. code-block:: bash
@@ -128,7 +184,7 @@ def _package_conf_file_to_dir(file_name):
                 return False
             else:
                 os.rename(path, path + '.tmpbak')
-                os.mkdir(path, 0755)
+                os.mkdir(path, 0o755)
                 with salt.utils.fopen(path + '.tmpbak') as fh_:
                     for line in fh_:
                         line = line.strip()
@@ -137,7 +193,7 @@ def _package_conf_file_to_dir(file_name):
                 os.remove(path + '.tmpbak')
                 return True
         else:
-            os.mkdir(path, 0755)
+            os.mkdir(path, 0o755)
             return True
 
 
@@ -201,12 +257,30 @@ def _package_conf_ordering(conf, clean=True, keep_backup=False):
                     shutil.rmtree(triplet[0])
 
 
-def _merge_flags(*args):
+def _check_accept_keywords(approved, flag):
+    '''check compatibility of accept_keywords'''
+    if flag in approved:
+        return False
+    elif (flag.startswith('~') and flag[1:] in approved) \
+            or ('~'+flag in approved):
+        return False
+    else:
+        return True
+
+
+def _merge_flags(new_flags, old_flags=None, conf='any'):
     '''
     Merges multiple lists of flags removing duplicates and resolving conflicts
     giving priority to lasts lists.
     '''
-    tmp = portage.flatten(args)
+    if not old_flags:
+        old_flags = []
+    args = [old_flags, new_flags]
+    if conf == 'accept_keywords':
+        tmp = new_flags + \
+            [i for i in old_flags if _check_accept_keywords(new_flags, i)]
+    else:
+        tmp = portage.flatten(args)
     flags = {}
     for flag in tmp:
         if flag[0] == '-':
@@ -214,11 +288,11 @@ def _merge_flags(*args):
         else:
             flags[flag] = True
     tmp = []
-    for k, v in flags.iteritems():
-        if v:
-            tmp.append(k)
+    for key, val in six.iteritems(flags):
+        if val:
+            tmp.append(key)
         else:
-            tmp.append('-' + k)
+            tmp.append('-' + key)
 
     # Next sort is just aesthetic, can be commented for a small performance
     # boost
@@ -250,7 +324,7 @@ def append_to_package_conf(conf, atom='', flags=None, string='', overwrite=False
             new_flags = list(flags)
         else:
             atom = string.strip().split()[0]
-            new_flags = portage.dep.strip_empty(string.strip().split(' '))[1:]
+            new_flags = [flag for flag in string.strip().split(' ') if flag][1:]
             if '/' not in atom:
                 atom = _p_to_cp(atom)
                 string = '{0} {1}'.format(atom, ' '.join(new_flags))
@@ -273,17 +347,10 @@ def append_to_package_conf(conf, atom='', flags=None, string='', overwrite=False
         # boost
         new_flags.sort(cmp=lambda x, y: cmp(x.lstrip('-'), y.lstrip('-')))
 
-        package_file = _p_to_cp(atom)
-        if not package_file:
-            return
-
-        psplit = package_file.split('/')
-        if len(psplit) == 2:
-            pdir = BASE_PATH.format(conf) + '/' + psplit[0]
-            if not os.path.exists(pdir):
-                os.mkdir(pdir, 0755)
-
-        complete_file_path = BASE_PATH.format(conf) + '/' + package_file
+        complete_file_path = _get_config_file(conf, atom)
+        pdir = os.path.dirname(complete_file_path)
+        if not os.path.exists(pdir):
+            os.makedirs(pdir, 0o755)
 
         try:
             shutil.copy(complete_file_path, complete_file_path + '.bak')
@@ -311,7 +378,7 @@ def append_to_package_conf(conf, atom='', flags=None, string='', overwrite=False
                     new_contents += string.strip() + '\n'
                     added = True
                 else:
-                    old_flags = portage.dep.strip_empty(l_strip.split(' '))[1:]
+                    old_flags = [flag for flag in l_strip.split(' ') if flag][1:]
                     if conf == 'accept_keywords':
                         if not old_flags:
                             new_contents += l
@@ -320,7 +387,7 @@ def append_to_package_conf(conf, atom='', flags=None, string='', overwrite=False
                             continue
                         elif not new_flags:
                             continue
-                    merged_flags = _merge_flags(old_flags, new_flags)
+                    merged_flags = _merge_flags(new_flags, old_flags, conf)
                     if merged_flags:
                         new_contents += '{0} {1}\n'.format(
                             atom, ' '.join(merged_flags))
@@ -373,10 +440,19 @@ def get_flags_from_package_conf(conf, atom):
         salt '*' portage_config.get_flags_from_package_conf license salt
     '''
     if conf in SUPPORTED_CONFS:
-        package_file = '{0}/{1}'.format(BASE_PATH.format(conf), _p_to_cp(atom))
+        package_file = _get_config_file(conf, atom)
         if '/' not in atom:
             atom = _p_to_cp(atom)
-        match_list = set(_porttree().dbapi.xmatch("match-all", atom))
+
+        has_wildcard = '*' in atom
+        if has_wildcard:
+            match_list = set(atom)
+        else:
+            try:
+                match_list = set(_porttree().dbapi.xmatch("match-all", atom))
+            except AttributeError:
+                return []
+
         flags = []
         try:
             file_handler = salt.utils.fopen(package_file)
@@ -386,9 +462,16 @@ def get_flags_from_package_conf(conf, atom):
             for line in file_handler:
                 line = line.strip()
                 line_package = line.split()[0]
-                line_list = _porttree().dbapi.xmatch("match-all", line_package)
-                if match_list.issubset(line_list):
-                    f_tmp = portage.dep.strip_empty(line.strip().split()[1:])
+
+                found_match = False
+                if has_wildcard:
+                    found_match = line_package == atom
+                else:
+                    line_list = _porttree().dbapi.xmatch("match-all", line_package)
+                    found_match = match_list.issubset(line_list)
+
+                if found_match:
+                    f_tmp = [flag for flag in line.strip().split(' ') if flag][1:]
                     if f_tmp:
                         flags.extend(f_tmp)
                     else:
@@ -458,8 +541,18 @@ def is_present(conf, atom):
         salt '*' portage_config.is_present unmask salt
     '''
     if conf in SUPPORTED_CONFS:
-        package_file = '{0}/{1}'.format(BASE_PATH.format(conf), _p_to_cp(atom))
-        match_list = set(_porttree().dbapi.xmatch("match-all", atom))
+        if not isinstance(atom, portage.dep.Atom):
+            atom = portage.dep.Atom(atom, allow_wildcard=True)
+        has_wildcard = '*' in atom
+
+        package_file = _get_config_file(conf, str(atom))
+
+        # wildcards are valid in confs
+        if has_wildcard:
+            match_list = set(atom)
+        else:
+            match_list = set(_porttree().dbapi.xmatch("match-all", atom))
+
         try:
             file_handler = salt.utils.fopen(package_file)
         except IOError:
@@ -468,7 +561,162 @@ def is_present(conf, atom):
             for line in file_handler:
                 line = line.strip()
                 line_package = line.split()[0]
-                line_list = _porttree().dbapi.xmatch("match-all", line_package)
-                if match_list.issubset(line_list):
-                    return True
+
+                if has_wildcard:
+                    if line_package == str(atom):
+                        return True
+                else:
+                    line_list = _porttree().dbapi.xmatch("match-all", line_package)
+                    if match_list.issubset(line_list):
+                        return True
             return False
+
+
+def get_iuse(cp):
+    '''
+    .. versionadded:: 2015.8.0
+
+    Gets the current IUSE flags from the tree.
+
+    @type: cpv: string
+    @param cpv: cat/pkg
+    @rtype list
+    @returns [] or the list of IUSE flags
+    '''
+    cpv = _get_cpv(cp)
+    try:
+        # aux_get might return dupes, so run them through set() to remove them
+        dirty_flags = _porttree().dbapi.aux_get(cpv, ["IUSE"])[0].split()
+        return list(set(dirty_flags))
+    except Exception as e:
+        return []
+
+
+def get_installed_use(cp, use="USE"):
+    '''
+    .. versionadded:: 2015.8.0
+
+    Gets the installed USE flags from the VARDB.
+
+    @type: cp: string
+    @param cp: cat/pkg
+    @type use: string
+    @param use: 1 of ["USE", "PKGUSE"]
+    @rtype list
+    @returns [] or the list of IUSE flags
+    '''
+    portage = _get_portage()
+    cpv = _get_cpv(cp)
+    return portage.db[portage.root]["vartree"].dbapi.aux_get(cpv, [use])[0].split()
+
+
+def filter_flags(use, use_expand_hidden, usemasked, useforced):
+    '''
+    .. versionadded:: 2015.8.0
+
+    Filter function to remove hidden or otherwise not normally
+    visible USE flags from a list.
+
+    @type use: list
+    @param use: the USE flag list to be filtered.
+    @type use_expand_hidden: list
+    @param  use_expand_hidden: list of flags hidden.
+    @type usemasked: list
+    @param usemasked: list of masked USE flags.
+    @type useforced: list
+    @param useforced: the forced USE flags.
+    @rtype: list
+    @return the filtered USE flags.
+    '''
+    portage = _get_portage()
+    # clean out some environment flags, since they will most probably
+    # be confusing for the user
+    for f in use_expand_hidden:
+        f = f.lower()+ "_"
+        for x in use:
+            if f in x:
+                use.remove(x)
+    # clean out any arch's
+    archlist = portage.settings["PORTAGE_ARCHLIST"].split()
+    for a in use[:]:
+        if a in archlist:
+            use.remove(a)
+    # dbl check if any from usemasked  or useforced are still there
+    masked = usemasked + useforced
+    for a in use[:]:
+        if a in masked:
+            use.remove(a)
+    return use
+
+
+def get_all_cpv_use(cp):
+    '''
+    .. versionadded:: 2015.8.0
+
+    Uses portage to determine final USE flags and settings for an emerge.
+
+    @type cp: string
+    @param cp: eg cat/pkg
+    @rtype: lists
+    @return  use, use_expand_hidden, usemask, useforce
+    '''
+    cpv = _get_cpv(cp)
+    portage = _get_portage()
+    use = None
+    _porttree().dbapi.settings.unlock()
+    try:
+        _porttree().dbapi.settings.setcpv(cpv, mydb=portage.portdb)
+        use = portage.settings['PORTAGE_USE'].split()
+        use_expand_hidden = portage.settings["USE_EXPAND_HIDDEN"].split()
+        usemask = list(_porttree().dbapi.settings.usemask)
+        useforce = list(_porttree().dbapi.settings.useforce)
+    except KeyError:
+        _porttree().dbapi.settings.reset()
+        _porttree().dbapi.settings.lock()
+        return [], [], [], []
+    # reset cpv filter
+    _porttree().dbapi.settings.reset()
+    _porttree().dbapi.settings.lock()
+    return use, use_expand_hidden, usemask, useforce
+
+
+def get_cleared_flags(cp):
+    '''
+    .. versionadded:: 2015.8.0
+
+    Uses portage for compare use flags which is used for installing package
+    and use flags which now exist int /etc/portage/package.use/
+
+    @type cp: string
+    @param cp: eg cat/pkg
+    @rtype: tuple
+    @rparam: tuple with two lists - list of used flags and
+    list of flags which will be used
+    '''
+    cpv = _get_cpv(cp)
+    final_use, use_expand_hidden, usemasked, useforced = get_all_cpv_use(cpv)
+    inst_flags = filter_flags(get_installed_use(cpv), use_expand_hidden,
+                                usemasked, useforced)
+    final_flags = filter_flags(final_use, use_expand_hidden,
+                                usemasked, useforced)
+    return inst_flags, final_flags
+
+
+def is_changed_uses(cp):
+    '''
+    .. versionadded:: 2015.8.0
+
+    Uses portage for determine if the use flags of installed package
+    is compatible with use flags in portage configs.
+
+    @type cp: string
+    @param cp: eg cat/pkg
+    '''
+    cpv = _get_cpv(cp)
+    i_flags, conf_flags = get_cleared_flags(cpv)
+    for i in i_flags:
+        try:
+            conf_flags.remove(i)
+        except ValueError:
+            return True
+    return True if conf_flags else False
