@@ -9,13 +9,12 @@ minion.
 from __future__ import absolute_import
 
 # Import python libs
-import os
-import shutil
-import signal
-import logging
-import fnmatch
-import sys
 import copy
+import fnmatch
+import logging
+import os
+import signal
+import sys
 
 # Import 3rd-party libs
 # pylint: disable=import-error
@@ -26,7 +25,7 @@ try:
 except ImportError:
     HAS_ESKY = False
 # pylint: disable=no-name-in-module
-from salt.ext.six import string_types
+from salt.ext import six
 from salt.ext.six.moves.urllib.error import URLError
 # pylint: enable=import-error,no-name-in-module
 
@@ -38,19 +37,22 @@ except AttributeError:
 
 # Import salt libs
 import salt
-import salt.payload
-import salt.state
+import salt.config
 import salt.client
 import salt.client.ssh.client
-import salt.config
+import salt.payload
 import salt.runner
-import salt.utils
-import salt.utils.process
-import salt.utils.minion
+import salt.state
 import salt.transport
+import salt.utils
+import salt.utils.event
+import salt.utils.extmods
+import salt.utils.minion
+import salt.utils.process
+import salt.utils.url
 import salt.wheel
 from salt.exceptions import (
-    SaltReqTimeoutError, SaltRenderError, CommandExecutionError
+    SaltReqTimeoutError, SaltRenderError, CommandExecutionError, SaltInvocationError
 )
 
 __proxyenabled__ = ['*']
@@ -86,104 +88,29 @@ def _sync(form, saltenv=None):
     '''
     if saltenv is None:
         saltenv = _get_top_file_envs()
-    if isinstance(saltenv, string_types):
+    if isinstance(saltenv, six.string_types):
         saltenv = saltenv.split(',')
-    ret = []
-    remote = set()
-    source = os.path.join('salt://_{0}'.format(form))
-    mod_dir = os.path.join(__opts__['extension_modules'], '{0}'.format(form))
-    if not os.path.isdir(mod_dir):
-        log.info('Creating module dir {0!r}'.format(mod_dir))
-        try:
-            os.makedirs(mod_dir)
-        except (IOError, OSError):
-            msg = 'Cannot create cache module directory {0}. Check permissions.'
-            log.error(msg.format(mod_dir))
-    for sub_env in saltenv:
-        log.info('Syncing {0} for environment {1!r}'.format(form, sub_env))
-        cache = []
-        log.info('Loading cache from {0}, for {1})'.format(source, sub_env))
-        # Grab only the desired files (.py, .pyx, .so)
-        cache.extend(
-            __salt__['cp.cache_dir'](
-                source, sub_env, include_pat=r'E@\.(pyx?|so)$'
-            )
-        )
-        local_cache_dir = os.path.join(
-                __opts__['cachedir'],
-                'files',
-                sub_env,
-                '_{0}'.format(form)
-                )
-        log.debug('Local cache dir: {0!r}'.format(local_cache_dir))
-        for fn_ in cache:
-            relpath = os.path.relpath(fn_, local_cache_dir)
-            relname = os.path.splitext(relpath)[0].replace(os.sep, '.')
-            remote.add(relpath)
-            dest = os.path.join(mod_dir, relpath)
-            log.info('Copying {0!r} to {1!r}'.format(fn_, dest))
-            if os.path.isfile(dest):
-                # The file is present, if the sum differs replace it
-                hash_type = __opts__.get('hash_type', 'md5')
-                src_digest = salt.utils.get_hash(fn_, hash_type)
-                dst_digest = salt.utils.get_hash(dest, hash_type)
-                if src_digest != dst_digest:
-                    # The downloaded file differs, replace!
-                    shutil.copyfile(fn_, dest)
-                    ret.append('{0}.{1}'.format(form, relname))
-            else:
-                dest_dir = os.path.dirname(dest)
-                if not os.path.isdir(dest_dir):
-                    os.makedirs(dest_dir)
-                shutil.copyfile(fn_, dest)
-                ret.append('{0}.{1}'.format(form, relname))
-
-    touched = bool(ret)
-    if __opts__.get('clean_dynamic_modules', True):
-        current = set(_listdir_recursively(mod_dir))
-        for fn_ in current - remote:
-            full = os.path.join(mod_dir, fn_)
-            if os.path.isfile(full):
-                touched = True
-                os.remove(full)
-        # Cleanup empty dirs
-        while True:
-            emptydirs = _list_emptydirs(mod_dir)
-            if not emptydirs:
-                break
-            for emptydir in emptydirs:
-                touched = True
-                shutil.rmtree(emptydir, ignore_errors=True)
+    ret, touched = salt.utils.extmods.sync(__opts__, form, saltenv=saltenv)
     # Dest mod_dir is touched? trigger reload if requested
     if touched:
         mod_file = os.path.join(__opts__['cachedir'], 'module_refresh')
         with salt.utils.fopen(mod_file, 'a+') as ofile:
             ofile.write('')
+    if form == 'grains' and \
+       __opts__.get('grains_cache') and \
+       os.path.isfile(os.path.join(__opts__['cachedir'], 'grains.cache.p')):
+        try:
+            os.remove(os.path.join(__opts__['cachedir'], 'grains.cache.p'))
+        except OSError:
+            log.error('Could not remove grains cache!')
     return ret
 
 
-def _listdir_recursively(rootdir):
-    file_list = []
-    for root, dirs, files in os.walk(rootdir):
-        for filename in files:
-            relpath = os.path.relpath(root, rootdir).strip('.')
-            file_list.append(os.path.join(relpath, filename))
-    return file_list
-
-
-def _list_emptydirs(rootdir):
-    emptydirs = []
-    for root, dirs, files in os.walk(rootdir):
-        if not files and not dirs:
-            emptydirs.append(root)
-    return emptydirs
-
-
 def update(version=None):
     '''
     Update the salt minion from the URL defined in opts['update_url']
     SaltStack, Inc provides the latest builds here:
-    update_url: http://docs.saltstack.com/downloads/
+    update_url: https://repo.saltstack.com/windows/
 
     Be aware that as of 2014-8-11 there's a bug in esky such that only the
     latest version available in the update_url can be downloaded and installed.
@@ -195,7 +122,7 @@ def update(version=None):
 
     Returns details about the transaction upon completion.
 
-    CLI Example:
+    CLI Examples:
 
     .. code-block:: bash
 
@@ -247,18 +174,25 @@ def update(version=None):
 
 def sync_beacons(saltenv=None, refresh=True):
     '''
-    Sync the beacons from the _beacons directory on the salt master file
-    server. This function is environment aware, pass the desired environment
-    to grab the contents of the _beacons directory, base is the default
-    environment.
-
     .. versionadded:: 2015.5.1
 
-    CLI Example:
+    Sync beacons from ``salt://_beacons`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available beacons on the minion. This refresh
+        will be performed even if no new beacons are synced. Set to ``False``
+        to prevent this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_beacons
+        salt '*' saltutil.sync_beacons saltenv=base,dev
     '''
     ret = _sync('beacons', saltenv)
     if refresh:
@@ -266,20 +200,65 @@ def sync_beacons(saltenv=None, refresh=True):
     return ret
 
 
+def sync_sdb(saltenv=None):
+    '''
+    .. versionadded:: 2015.5.8,2015.8.3
+
+    Sync sdb modules from ``salt://_sdb`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_sdb
+        salt '*' saltutil.sync_sdb saltenv=base,dev
+    '''
+    ret = _sync('sdb', saltenv)
+    return ret
+
+
 def sync_modules(saltenv=None, refresh=True):
     '''
-    Sync the modules from the _modules directory on the salt master file
-    server. This function is environment aware, pass the desired environment
-    to grab the contents of the _modules directory, base is the default
-    environment.
+    .. versionadded:: 0.10.0
 
-    .. versionadded:: 2015.5.1
+    Sync execution modules from ``salt://_modules`` to the minion
 
-    CLI Example:
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new execution modules are
+        synced. Set to ``False`` to prevent this refresh.
+
+    .. important::
+
+        If this function is executed using a :py:func:`module.run
+        <salt.states.module.run>` state, the SLS file will not have access to
+        newly synced execution modules unless a ``refresh`` argument is
+        added to the state, like so:
+
+        .. code-block:: yaml
+
+            load_my_custom_module:
+              module.run:
+                - name: saltutil.sync_modules
+                - refresh: True
+
+        See :ref:`here <reloading-modules>` for a more detailed explanation of
+        why this is necessary.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_modules
+        salt '*' saltutil.sync_modules saltenv=base,dev
     '''
     ret = _sync('modules', saltenv)
     if refresh:
@@ -289,16 +268,25 @@ def sync_modules(saltenv=None, refresh=True):
 
 def sync_states(saltenv=None, refresh=True):
     '''
-    Sync the states from the _states directory on the salt master file
-    server. This function is environment aware, pass the desired environment
-    to grab the contents of the _states directory, base is the default
-    environment.
+    .. versionadded:: 0.10.0
 
-    CLI Example:
+    Sync state modules from ``salt://_states`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available states on the minion. This refresh
+        will be performed even if no new state modules are synced. Set to
+        ``False`` to prevent this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_states
+        salt '*' saltutil.sync_states saltenv=base,dev
     '''
     ret = _sync('states', saltenv)
     if refresh:
@@ -308,16 +296,26 @@ def sync_states(saltenv=None, refresh=True):
 
 def sync_grains(saltenv=None, refresh=True):
     '''
-    Sync the grains from the _grains directory on the salt master file
-    server. This function is environment aware, pass the desired environment
-    to grab the contents of the _grains directory, base is the default
-    environment.
+    .. versionadded:: 0.10.0
 
-    CLI Example:
+    Sync grains modules from ``salt://_grains`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules and recompile
+        pillar data for the minion. This refresh will be performed even if no
+        new grains modules are synced. Set to ``False`` to prevent this
+        refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_grains
+        salt '*' saltutil.sync_grains saltenv=base,dev
     '''
     ret = _sync('grains', saltenv)
     if refresh:
@@ -328,16 +326,26 @@ def sync_grains(saltenv=None, refresh=True):
 
 def sync_renderers(saltenv=None, refresh=True):
     '''
-    Sync the renderers from the _renderers directory on the salt master file
-    server. This function is environment aware, pass the desired environment
-    to grab the contents of the _renderers directory, base is the default
-    environment.
+    .. versionadded:: 0.10.0
 
-    CLI Example:
+    Sync renderers from ``salt://_renderers`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new renderers are synced.
+        Set to ``False`` to prevent this refresh. Set to ``False`` to prevent
+        this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_renderers
+        salt '*' saltutil.sync_renderers saltenv=base,dev
     '''
     ret = _sync('renderers', saltenv)
     if refresh:
@@ -347,12 +355,20 @@ def sync_renderers(saltenv=None, refresh=True):
 
 def sync_returners(saltenv=None, refresh=True):
     '''
-    Sync the returners from the _returners directory on the salt master file
-    server. This function is environment aware, pass the desired environment
-    to grab the contents of the _returners directory, base is the default
-    environment.
+    .. versionadded:: 0.10.0
 
-    CLI Example:
+    Sync beacons from ``salt://_returners`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new returners are synced. Set
+        to ``False`` to prevent this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
@@ -364,37 +380,111 @@ def sync_returners(saltenv=None, refresh=True):
     return ret
 
 
-def sync_outputters(saltenv=None, refresh=True):
+def sync_proxymodules(saltenv=None, refresh=False):
     '''
-    Sync the outputters from the _outputters directory on the salt master file
-    server. This function is environment aware, pass the desired environment
-    to grab the contents of the _outputters directory, base is the default
-    environment.
+    .. versionadded:: 2015.8.2
 
-    CLI Example:
+    Sync proxy modules from ``salt://_proxy`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new proxy modules are synced.
+        Set to ``False`` to prevent this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
-        salt '*' saltutil.sync_outputters
+        salt '*' saltutil.sync_proxymodules
+        salt '*' saltutil.sync_proxymodules saltenv=base,dev
     '''
-    ret = _sync('outputters', saltenv)
+    ret = _sync('proxy', saltenv)
     if refresh:
         refresh_modules()
     return ret
 
 
+def sync_engines(saltenv=None, refresh=False):
+    '''
+    .. versionadded:: 2016.3.0
+
+    Sync engine modules from ``salt://_engines`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new engine modules are synced.
+        Set to ``False`` to prevent this refresh.
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_engines
+        salt '*' saltutil.sync_engines saltenv=base,dev
+    '''
+    ret = _sync('engines', saltenv)
+    if refresh:
+        refresh_modules()
+    return ret
+
+
+def sync_output(saltenv=None, refresh=True):
+    '''
+    Sync outputters from ``salt://_output`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new outputters are synced.
+        Set to ``False`` to prevent this refresh.
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_output
+        salt '*' saltutil.sync_output saltenv=base,dev
+    '''
+    ret = _sync('output', saltenv)
+    if refresh:
+        refresh_modules()
+    return ret
+
+sync_outputters = salt.utils.alias_function(sync_output, 'sync_outputters')
+
+
 def sync_utils(saltenv=None, refresh=True):
     '''
-    Sync utility source files from the _utils directory on the salt master file
-    server. This function is environment aware, pass the desired environment
-    to grab the contents of the _utils directory, base is the default
-    environment.
+    .. versionadded:: 2014.7.0
 
-    CLI Example:
+    Sync utility modules from ``salt://_utils`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new utility modules are
+        synced. Set to ``False`` to prevent this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_utils
+        salt '*' saltutil.sync_utils saltenv=base,dev
     '''
     ret = _sync('utils', saltenv)
     if refresh:
@@ -402,32 +492,86 @@ def sync_utils(saltenv=None, refresh=True):
     return ret
 
 
+def sync_log_handlers(saltenv=None, refresh=True):
+    '''
+    .. versionadded:: 2015.8.0
+
+    Sync log handlers from ``salt://_log_handlers`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new log handlers are synced.
+        Set to ``False`` to prevent this refresh.
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_log_handlers
+        salt '*' saltutil.sync_log_handlers saltenv=base,dev
+    '''
+    ret = _sync('log_handlers', saltenv)
+    if refresh:
+        refresh_modules()
+    return ret
+
+
 def sync_all(saltenv=None, refresh=True):
     '''
     Sync down all of the dynamic modules from the file server for a specific
-    environment
+    environment. This function synchronizes custom modules, states, beacons,
+    grains, returners, output modules, renderers, and utils.
 
     refresh : True
-        Also refresh the execution modules available to the minion.
+        Also refresh the execution modules and recompile pillar data available
+        to the minion. This refresh will be performed even if no new dynamic
+        modules are synced. Set to ``False`` to prevent this refresh.
 
-    CLI Example:
+    .. important::
+
+        If this function is executed using a :py:func:`module.run
+        <salt.states.module.run>` state, the SLS file will not have access to
+        newly synced execution modules unless a ``refresh`` argument is
+        added to the state, like so:
+
+        .. code-block:: yaml
+
+            load_my_custom_module:
+              module.run:
+                - name: saltutil.sync_all
+                - refresh: True
+
+        See :ref:`here <reloading-modules>` for a more detailed explanation of
+        why this is necessary.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_all
+        salt '*' saltutil.sync_all saltenv=base,dev
     '''
     log.debug('Syncing all')
     ret = {}
     ret['beacons'] = sync_beacons(saltenv, False)
     ret['modules'] = sync_modules(saltenv, False)
     ret['states'] = sync_states(saltenv, False)
+    ret['sdb'] = sync_sdb(saltenv)
     ret['grains'] = sync_grains(saltenv, False)
     ret['renderers'] = sync_renderers(saltenv, False)
     ret['returners'] = sync_returners(saltenv, False)
-    ret['outputters'] = sync_outputters(saltenv, False)
+    ret['output'] = sync_output(saltenv, False)
     ret['utils'] = sync_utils(saltenv, False)
+    ret['log_handlers'] = sync_log_handlers(saltenv, False)
+    ret['proxymodules'] = sync_proxymodules(saltenv, False)
+    ret['engines'] = sync_engines(saltenv, False)
     if refresh:
         refresh_modules()
+        refresh_pillar()
     return ret
 
 
@@ -466,13 +610,17 @@ def refresh_pillar():
         ret = False  # Effectively a no-op, since we can't really return without an event system
     return ret
 
-pillar_refresh = refresh_pillar
+pillar_refresh = salt.utils.alias_function(refresh_pillar, 'pillar_refresh')
 
 
-def refresh_modules():
+def refresh_modules(async=True):
     '''
     Signal the minion to refresh the module and grain data
 
+    The default is to refresh module asynchronously. To block
+    until the module refresh is complete, set the 'async' flag
+    to False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -480,7 +628,16 @@ def refresh_modules():
         salt '*' saltutil.refresh_modules
     '''
     try:
-        ret = __salt__['event.fire']({}, 'module_refresh')
+        if async:
+            #  If we're going to block, first setup a listener
+            ret = __salt__['event.fire']({}, 'module_refresh')
+        else:
+            eventer = salt.utils.event.get_event('minion', opts=__opts__, listen=True)
+            ret = __salt__['event.fire']({'notify': True}, 'module_refresh')
+            # Wait for the finish event to fire
+            log.trace('refresh_modules waiting for module refresh to complete')
+            # Blocks until we hear this event or until the timeout expires
+            eventer.get_event(tag='/salt/minion/minion_mod_complete', wait=30)
     except KeyError:
         log.error('Event module not available. Module refresh failed.')
         ret = False  # Effectively a no-op, since we can't really return without an event system
@@ -546,13 +703,49 @@ def clear_cache():
 
 def find_job(jid):
     '''
-    Return the data for a specific job id
+    Return the data for a specific job id that is currently running.
+
+    jid
+        The job id to search for and return data.
 
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' saltutil.find_job <job id>
+
+    Note that the find_job function only returns job information when the job is still running. If
+    the job is currently running, the output looks something like this:
+
+    .. code-block:: bash
+
+        # salt my-minion saltutil.find_job 20160503150049487736
+        my-minion:
+            ----------
+            arg:
+                - 30
+            fun:
+                test.sleep
+            jid:
+                20160503150049487736
+            pid:
+                9601
+            ret:
+            tgt:
+                my-minion
+            tgt_type:
+                glob
+            user:
+                root
+
+    If the job has already completed, the job cannot be found and therefore the function returns
+    an empty dictionary, which looks like this on the CLI:
+
+    .. code-block:: bash
+
+        # salt my-minion saltutil.find_job 20160503150049487736
+        my-minion:
+            ----------
     '''
     for data in running():
         if data['jid'] == jid:
@@ -562,7 +755,8 @@ def find_job(jid):
 
 def find_cached_job(jid):
     '''
-    Return the data for a specific cached job id
+    Return the data for a specific cached job id. Note this only works if
+    cache_jobs has previously been set to True on the minion.
 
     CLI Example:
 
@@ -574,7 +768,11 @@ def find_cached_job(jid):
     proc_dir = os.path.join(__opts__['cachedir'], 'minion_jobs')
     job_dir = os.path.join(proc_dir, str(jid))
     if not os.path.isdir(job_dir):
-        return
+        if not __opts__.get('cache_jobs'):
+            return ('Local jobs cache directory not found; you may need to'
+                    ' enable cache_jobs on this minion')
+        else:
+            return 'Local jobs cache directory {0} not found'.format(job_dir)
     path = os.path.join(job_dir, 'return.p')
     with salt.utils.fopen(path, 'rb') as fp_:
         buf = fp_.read()
@@ -710,18 +908,31 @@ def _get_ssh_or_api_client(cfgfile, ssh=False):
 
 
 def _exec(client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs):
-    ret = {}
+    fcn_ret = {}
     seen = 0
-    for ret_comp in client.cmd_iter(
-            tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs):
-        ret.update(ret_comp)
+    if 'batch' in kwargs:
+        _cmd = client.cmd_batch
+        cmd_kwargs = {
+            'tgt': tgt, 'fun': fun, 'arg': arg, 'expr_form': expr_form,
+            'ret': ret, 'kwarg': kwarg, 'batch': kwargs['batch'],
+        }
+        del kwargs['batch']
+    else:
+        _cmd = client.cmd_iter
+        cmd_kwargs = {
+            'tgt': tgt, 'fun': fun, 'arg': arg, 'timeout': timeout,
+            'expr_form': expr_form, 'ret': ret, 'kwarg': kwarg,
+        }
+    cmd_kwargs.update(kwargs)
+    for ret_comp in _cmd(**cmd_kwargs):
+        fcn_ret.update(ret_comp)
         seen += 1
-        # ret can be empty, so we cannot len the whole return dict
+        # fcn_ret can be empty, so we cannot len the whole return dict
         if expr_form == 'list' and len(tgt) == seen:
             # do not wait for timeout when explicit list matching
             # and all results are there
             break
-    return ret
+    return fcn_ret
 
 
 def cmd(tgt,
@@ -744,21 +955,30 @@ def cmd(tgt,
     '''
     cfgfile = __opts__['conf_file']
     client = _get_ssh_or_api_client(cfgfile, ssh)
-    ret = _exec(
+    fcn_ret = _exec(
         client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
     # if return is empty, we may have not used the right conf,
     # try with the 'minion relative master configuration counter part
     # if available
     master_cfgfile = '{0}master'.format(cfgfile[:-6])  # remove 'minion'
     if (
-        not ret
+        not fcn_ret
         and cfgfile.endswith('{0}{1}'.format(os.path.sep, 'minion'))
         and os.path.exists(master_cfgfile)
     ):
         client = _get_ssh_or_api_client(master_cfgfile, ssh)
-        ret = _exec(
+        fcn_ret = _exec(
             client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
-    return ret
+
+    if 'batch' in kwargs:
+        old_ret, fcn_ret = fcn_ret, {}
+        for key, value in old_ret.items():
+            fcn_ret[key] = {
+                'out': value.get('out', 'highstate') if isinstance(value, dict) else 'highstate',
+                'ret': value,
+            }
+
+    return fcn_ret
 
 
 def cmd_iter(tgt,
@@ -795,14 +1015,15 @@ def cmd_iter(tgt,
         yield ret
 
 
-def runner(fun, **kwargs):
+def runner(_fun, **kwargs):
     '''
     Execute a runner module (this function must be run on the master)
 
-    .. versionadded:: 2014.7
+    .. versionadded:: 2014.7.0
 
-    name
+    _fun
         The name of the function to run
+
     kwargs
         Any keyword arguments to pass to the runner function
 
@@ -822,17 +1043,24 @@ def runner(fun, **kwargs):
     else:
         rclient = salt.runner.RunnerClient(__opts__)
 
-    return rclient.cmd(fun, [], kwarg=kwargs)
+    return rclient.cmd(_fun, kwarg=kwargs)
 
 
-def wheel(fun, **kwargs):
+def wheel(_fun, *args, **kwargs):
     '''
     Execute a wheel module (this function must be run on the master)
 
-    .. versionadded:: 2014.7
+    .. versionadded:: 2014.7.0
 
     name
         The name of the function to run
+
+    args
+        Any positional arguments to pass to the wheel function. A common example
+        of this would be the ``match`` arg needed for key functions.
+
+        .. versionadded:: v2015.8.11
+
     kwargs
         Any keyword arguments to pass to the wheel function
 
@@ -840,10 +1068,33 @@ def wheel(fun, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' saltutil.wheel key.accept match=jerry
+        salt '*' saltutil.wheel key.accept jerry
     '''
-    wclient = salt.wheel.WheelClient(__opts__)
-    return wclient.cmd(fun, kwarg=kwargs)
+    if __opts__['__role'] == 'minion':
+        master_config = os.path.join(os.path.dirname(__opts__['conf_file']),
+                                     'master')
+        master_opts = salt.config.client_config(master_config)
+        wheel_client = salt.wheel.WheelClient(master_opts)
+    else:
+        wheel_client = salt.wheel.WheelClient(__opts__)
+
+    # The WheelClient cmd needs args, kwargs, and pub_data separated out from
+    # the "normal" kwargs structure, which at this point contains __pub_x keys.
+    pub_data = {}
+    valid_kwargs = {}
+    for key, val in six.iteritems(kwargs):
+        if key.startswith('__'):
+            pub_data[key] = val
+        else:
+            valid_kwargs[key] = val
+
+    try:
+        ret = wheel_client.cmd(_fun, arg=args, pub_data=pub_data, kwarg=valid_kwargs)
+    except SaltInvocationError:
+        raise CommandExecutionError('This command can only be executed on a minion '
+                                    'that is located on the master.')
+
+    return ret
 
 
 # this is the only way I could figure out how to get the REAL file_roots
