@@ -6,6 +6,7 @@ uWSGI stats server http://uwsgi-docs.readthedocs.org/en/latest/StatsServer.html
 :maturity:   new
 :platform:   all
 '''
+from __future__ import absolute_import
 
 # Import Python libs
 import json
@@ -21,7 +22,7 @@ def __virtual__():
     cmd = 'uwsgi'
     if salt.utils.which(cmd):
         return cmd
-    return False
+    return (False, 'The uwsgi execution module failed to load: the uwsgi binary is not in the path.')
 
 
 def stats(socket):
