@@ -262,6 +262,9 @@ Links
 For more examples and information see <https://github.com/mickep76/pepa>.
 '''
 
+# Import futures
+from __future__ import absolute_import, print_function
+
 __author__ = 'Michael Persson <michael.ake.persson@gmail.com>'
 __copyright__ = 'Copyright (c) 2013 Michael Persson'
 __license__ = 'Apache License, Version 2.0'
@@ -276,13 +279,23 @@ import jinja2
 import re
 from os.path import isfile, join
 
+# Import 3rd-party libs
+import salt.ext.six as six
+from salt.ext.six.moves import input  # pylint: disable=import-error,redefined-builtin
+
+try:
+    import requests
+    HAS_REQUESTS = True
+except ImportError:
+    HAS_REQUESTS = False
+
 # Import Salt libs
 import salt.utils
 
 # Only used when called from a terminal
 log = None
 if __name__ == '__main__':
-    import argparse
+    import argparse  # pylint: disable=minimum-python-version
 
     parser = argparse.ArgumentParser()
     parser.add_argument('hostname', help='Hostname')
@@ -305,7 +318,7 @@ if __name__ == '__main__':
     formatter = None
     if not args.no_color:
         try:
-            import colorlog
+            import colorlog  # pylint: disable=import-error
             formatter = colorlog.ColoredFormatter("[%(log_color)s%(levelname)-8s%(reset)s] %(log_color)s%(message)s%(reset)s")
         except ImportError:
             formatter = logging.Formatter("[%(levelname)-8s] %(message)s")
@@ -337,6 +350,9 @@ def __virtual__():
     '''
     Only return if all the modules are available
     '''
+    if not HAS_REQUESTS:
+        return False
+
     return True
 
 
@@ -345,7 +361,7 @@ def key_value_to_tree(data):
     Convert key/value to tree
     '''
     tree = {}
-    for flatkey, value in data.items():
+    for flatkey, value in six.iteritems(data):
         t = tree
         keys = flatkey.split(__opts__['pepa_delimiter'])
         for key in keys:
@@ -379,7 +395,7 @@ def ext_pillar(minion_id, pillar, resource, sequence, subkey=False, subkey_only=
     output['pepa_templates'] = []
     immutable = {}
 
-    for categ, info in [s.items()[0] for s in sequence]:
+    for categ, info in [next(six.iteritems(s)) for s in sequence]:
         if categ not in inp:
             log.warn("Category is not defined: {0}".format(categ))
             continue
@@ -421,9 +437,9 @@ def ext_pillar(minion_id, pillar, resource, sequence, subkey=False, subkey_only=
                     data['pillar'] = pillar.copy()
                     results_jinja = template.render(data)
                     results = yaml.load(results_jinja)
-                except jinja2.UndefinedError, err:
+                except jinja2.UndefinedError as err:
                     log.error('Failed to parse JINJA template: {0}\n{1}'.format(fn, err))
-                except yaml.YAMLError, err:
+                except yaml.YAMLError as err:
                     log.error('Failed to parse YAML in template: {0}\n{1}'.format(fn, err))
             else:
                 log.info("Template doesn't exist: {0}".format(fn))
@@ -450,11 +466,11 @@ def ext_pillar(minion_id, pillar, resource, sequence, subkey=False, subkey_only=
                             immutable[rkey] = True
                         if rkey not in output:
                             log.error('Cant\'t merge key {0} doesn\'t exist'.format(rkey))
-                        elif type(results[key]) != type(output[rkey]):
+                        elif not isinstance(results[key], type(output[rkey])):
                             log.error('Can\'t merge different types for key {0}'.format(rkey))
-                        elif type(results[key]) is dict:
+                        elif isinstance(results[key], dict):
                             output[rkey].update(results[key])
-                        elif type(results[key]) is list:
+                        elif isinstance(results[key], list):
                             output[rkey].extend(results[key])
                         else:
                             log.error('Unsupported type need to be list or dict for key {0}'.format(rkey))
@@ -495,7 +511,7 @@ def validate(output, resource):
     Validate Pepa templates
     '''
     try:
-        import cerberus
+        import cerberus  # pylint: disable=import-error
     except ImportError:
         log.critical('You need module cerberus in order to use validation')
         return
@@ -519,7 +535,7 @@ def validate(output, resource):
 
     val = cerberus.Validator()
     if not val.validate(output['pepa_keys'], all_schemas):
-        for ekey, error in val.errors.items():
+        for ekey, error in six.iteritems(val.errors):
             log.warning('Validation failed for key {0}: {1}'.format(ekey, error))
 
     output['pepa_schema_keys'] = all_schemas
@@ -538,7 +554,7 @@ if __name__ == '__main__':
         __opts__.update(yaml.load(fh_.read()))
 
     loc = 0
-    for name in [e.iterkeys().next() for e in __opts__['ext_pillar']]:
+    for name in [next(iter(list(e.keys()))) for e in __opts__['ext_pillar']]:
         if name == 'pepa':
             break
         loc += 1
@@ -568,7 +584,7 @@ if __name__ == '__main__':
         username = args.username
         password = args.password
         if username is None:
-            username = raw_input('Username: ')
+            username = input('Username: ')
         if password is None:
             password = getpass.getpass()
 
@@ -610,11 +626,16 @@ if __name__ == '__main__':
     yaml.dumper.SafeDumper.ignore_aliases = lambda self, data: True
     if not args.no_color:
         try:
+            # pylint: disable=import-error
             import pygments
             import pygments.lexers
             import pygments.formatters
-            print pygments.highlight(yaml.safe_dump(result), pygments.lexers.YamlLexer(), pygments.formatters.TerminalFormatter())
+            # pylint: disable=no-member
+            print(pygments.highlight(yaml.safe_dump(result),
+                                     pygments.lexers.YamlLexer(),
+                                     pygments.formatters.TerminalFormatter()))
+            # pylint: enable=no-member, import-error
         except ImportError:
-            print yaml.safe_dump(result, indent=4, default_flow_style=False)
+            print(yaml.safe_dump(result, indent=4, default_flow_style=False))
     else:
-        print yaml.safe_dump(result, indent=4, default_flow_style=False)
+        print(yaml.safe_dump(result, indent=4, default_flow_style=False))
