@@ -2,12 +2,21 @@
 '''
 Nova class
 '''
-from __future__ import with_statement
+
+# Import Python libs
+from __future__ import absolute_import, with_statement
+from distutils.version import LooseVersion
+import time
+import inspect
+import logging
 
 # Import third party libs
+import salt.ext.six as six
 HAS_NOVA = False
+# pylint: disable=import-error
 try:
-    from novaclient.v1_1 import client
+    import novaclient
+    from novaclient import client
     from novaclient.shell import OpenStackComputeShell
     import novaclient.utils
     import novaclient.auth_plugin
@@ -17,11 +26,7 @@ try:
     HAS_NOVA = True
 except ImportError:
     pass
-
-# Import python libs
-import inspect
-import time
-import logging
+# pylint: enable=import-error
 
 # Import salt libs
 import salt.utils
@@ -30,9 +35,32 @@ from salt.exceptions import SaltCloudSystemExit
 # Get logging started
 log = logging.getLogger(__name__)
 
+# Version added to novaclient.client.Client function
+NOVACLIENT_MINVER = '2.6.1'
+
+# dict for block_device_mapping_v2
+CLIENT_BDM2_KEYS = {
+    'id': 'uuid',
+    'source': 'source_type',
+    'dest': 'destination_type',
+    'bus': 'disk_bus',
+    'device': 'device_name',
+    'size': 'volume_size',
+    'format': 'guest_format',
+    'bootindex': 'boot_index',
+    'type': 'device_type',
+    'shutdown': 'delete_on_termination',
+}
+
 
 def check_nova():
-    return HAS_NOVA
+    if HAS_NOVA:
+        novaclient_ver = LooseVersion(novaclient.__version__)
+        min_ver = LooseVersion(NOVACLIENT_MINVER)
+        if novaclient_ver >= min_ver:
+            return HAS_NOVA
+        log.debug('Newer novaclient version required.  Minimum: {0}'.format(NOVACLIENT_MINVER))
+    return False
 
 
 # kwargs has to be an object instead of a dictionary for the __post_parse_arg__
@@ -41,6 +69,60 @@ class KwargsStruct(object):
         self.__dict__.update(entries)
 
 
+def _parse_block_device_mapping_v2(block_device=None, boot_volume=None, snapshot=None, ephemeral=None, swap=None):
+    bdm = []
+    if block_device is None:
+        block_device = []
+    if ephemeral is None:
+        ephemeral = []
+
+    if boot_volume is not None:
+        bdm_dict = {'uuid': boot_volume, 'source_type': 'volume',
+                    'destination_type': 'volume', 'boot_index': 0,
+                    'delete_on_termination': False}
+        bdm.append(bdm_dict)
+
+    if snapshot is not None:
+        bdm_dict = {'uuid': snapshot, 'source_type': 'snapshot',
+                    'destination_type': 'volume', 'boot_index': 0,
+                    'delete_on_termination': False}
+        bdm.append(bdm_dict)
+
+    for device_spec in block_device:
+        bdm_dict = {}
+
+        for key, value in six.iteritems(device_spec):
+            bdm_dict[CLIENT_BDM2_KEYS[key]] = value
+
+        # Convert the delete_on_termination to a boolean or set it to true by
+        # default for local block devices when not specified.
+        if 'delete_on_termination' in bdm_dict:
+            action = bdm_dict['delete_on_termination']
+            bdm_dict['delete_on_termination'] = (action == 'remove')
+        elif bdm_dict.get('destination_type') == 'local':
+            bdm_dict['delete_on_termination'] = True
+
+        bdm.append(bdm_dict)
+
+    for ephemeral_spec in ephemeral:
+        bdm_dict = {'source_type': 'blank', 'destination_type': 'local',
+                    'boot_index': -1, 'delete_on_termination': True}
+        if 'size' in ephemeral_spec:
+            bdm_dict['volume_size'] = ephemeral_spec['size']
+        if 'format' in ephemeral_spec:
+            bdm_dict['guest_format'] = ephemeral_spec['format']
+
+        bdm.append(bdm_dict)
+
+    if swap is not None:
+        bdm_dict = {'source_type': 'blank', 'destination_type': 'local',
+                    'boot_index': -1, 'delete_on_termination': True,
+                    'guest_format': 'swap', 'volume_size': swap}
+        bdm.append(bdm_dict)
+
+    return bdm
+
+
 class NovaServer(object):
     def __init__(self, name, server, password=None):
         '''
@@ -48,7 +130,7 @@ class NovaServer(object):
         '''
         self.name = name
         self.id = server['id']
-        self.image = server['image']['id']
+        self.image = server.get('image', {}).get('id', 'Boot From Volume')
         self.size = server['flavor']['id']
         self.state = server['state']
         self._uuid = None
@@ -57,22 +139,14 @@ class NovaServer(object):
             'access_ip': server['accessIPv4']
         }
 
-        if 'addresses' in server:
-            if 'public' in server['addresses']:
-                self.public_ips = [
-                    ip['addr'] for ip in server['addresses']['public']
-                ]
-            else:
-                self.public_ips = []
-
-            if 'private' in server['addresses']:
-                self.private_ips = [
-                    ip['addr'] for ip in server['addresses']['private']
-                ]
-            else:
-                self.private_ips = []
-
-            self.addresses = server['addresses']
+        self.addresses = server.get('addresses', {})
+        self.public_ips, self.private_ips = [], []
+        for network in self.addresses.values():
+            for addr in network:
+                if salt.utils.cloud.is_public_ip(addr['addr']):
+                    self.public_ips.append(addr['addr'])
+                else:
+                    self.private_ips.append(addr['addr'])
 
         if password:
             self.extra['password'] = password
@@ -81,11 +155,13 @@ class NovaServer(object):
         return self.__dict__
 
 
-def get_entry(dict_, key, value):
+def get_entry(dict_, key, value, raise_error=True):
     for entry in dict_:
         if entry[key] == value:
             return entry
-    raise SaltCloudSystemExit('Unable to find {0} in {1}.'.format(key, dict_))
+    if raise_error is True:
+        raise SaltCloudSystemExit('Unable to find {0} in {1}.'.format(key, dict_))
+    return {}
 
 
 def sanatize_novaclient(kwargs):
@@ -106,10 +182,12 @@ def sanatize_novaclient(kwargs):
 
 
 # Function alias to not shadow built-ins
-class SaltNova(OpenStackComputeShell):
+class SaltNova(object):
     '''
     Class for all novaclient functions
     '''
+    extensions = []
+
     def __init__(
         self,
         username,
@@ -123,13 +201,12 @@ class SaltNova(OpenStackComputeShell):
         '''
         Set up nova credentials
         '''
-        if not HAS_NOVA:
-            return None
-
         self.kwargs = kwargs.copy()
-
-        if not novaclient.base.Manager._hooks_map:
-            self.extensions = self._discover_extensions('1.1')
+        if not self.extensions:
+            if hasattr(OpenStackComputeShell, '_discover_extensions'):
+                self.extensions = OpenStackComputeShell()._discover_extensions('2.0')
+            else:
+                self.extensions = client.discover_extensions('2.0')
             for extension in self.extensions:
                 extension.run_hooks('__pre_parse_args__')
             self.kwargs['extensions'] = self.extensions
@@ -163,20 +240,19 @@ class SaltNova(OpenStackComputeShell):
 
         self.kwargs = sanatize_novaclient(self.kwargs)
 
-        if not hasattr(client.Client, '__exit__'):
-            raise SaltCloudSystemExit("Newer version of novaclient required for __exit__.")
+        # Requires novaclient version >= 2.6.1
+        self.kwargs['version'] = str(kwargs.get('version', 2))
 
-        with client.Client(**self.kwargs) as conn:
-            try:
-                conn.client.authenticate()
-            except novaclient.exceptions.AmbiguousEndpoints:
-                raise SaltCloudSystemExit(
-                    "Nova provider requires a 'region_name' to be specified"
-                )
+        conn = client.Client(**self.kwargs)
+        try:
+            conn.client.authenticate()
+        except novaclient.exceptions.AmbiguousEndpoints:
+            raise SaltCloudSystemExit(
+                "Nova provider requires a 'region_name' to be specified"
+            )
 
-            self.kwargs['auth_token'] = conn.client.auth_token
-            self.catalog = \
-                conn.client.service_catalog.catalog['access']['serviceCatalog']
+        self.kwargs['auth_token'] = conn.client.auth_token
+        self.catalog = conn.client.service_catalog.catalog['access']['serviceCatalog']
 
         if region_name is not None:
             servers_endpoints = get_entry(self.catalog, 'type', 'compute')['endpoints']
@@ -188,34 +264,34 @@ class SaltNova(OpenStackComputeShell):
 
         self.compute_conn = client.Client(**self.kwargs)
 
-        if region_name is not None:
-            servers_endpoints = get_entry(
-                self.catalog,
-                'type',
-                'volume'
-            )['endpoints']
-            self.kwargs['bypass_url'] = get_entry(
-                servers_endpoints,
-                'region',
-                region_name
-            )['publicURL']
+        volume_endpoints = get_entry(self.catalog, 'type', 'volume', raise_error=False).get('endpoints', {})
+        if volume_endpoints:
+            if region_name is not None:
+                self.kwargs['bypass_url'] = get_entry(
+                    volume_endpoints,
+                    'region',
+                    region_name
+                )['publicURL']
 
-        self.kwargs['service_type'] = 'volume'
-        self.volume_conn = client.Client(**self.kwargs)
-        if hasattr(self, 'extensions'):
-            self.expand_extensions()
+            self.volume_conn = client.Client(**self.kwargs)
+            if hasattr(self, 'extensions'):
+                self.expand_extensions()
+        else:
+            self.volume_conn = None
 
     def expand_extensions(self):
         for connection in (self.compute_conn, self.volume_conn):
+            if connection is None:
+                continue
             for extension in self.extensions:
                 for attr in extension.module.__dict__:
                     if not inspect.isclass(getattr(extension.module, attr)):
                         continue
-                    for key, value in connection.__dict__.items():
+                    for key, value in six.iteritems(connection.__dict__):
                         if not isinstance(value, novaclient.base.Manager):
                             continue
                         if value.__class__.__name__ == attr:
-                            setattr(connection, key, getattr(connection, extension.name))
+                            setattr(connection, key, extension.manager_class(connection))
 
     def get_catalog(self):
         '''
@@ -228,8 +304,8 @@ class SaltNova(OpenStackComputeShell):
         Make output look like libcloud output for consistency
         '''
         server_info = self.server_show(uuid)
-        server = server_info.itervalues().next()
-        server_name = server_info.iterkeys().next()
+        server = next(six.itervalues(server_info))
+        server_name = next(six.iterkeys(server_info))
         if not hasattr(self, 'password'):
             self.password = None
         ret = NovaServer(server_name, server, self.password)
@@ -241,14 +317,21 @@ class SaltNova(OpenStackComputeShell):
         Boot a cloud server.
         '''
         nt_ks = self.compute_conn
-        for key in ('name', 'flavor', 'image'):
-            if key in kwargs:
-                del kwargs[key]
-        response = nt_ks.servers.create(
-            name=name, flavor=flavor_id, image=image_id, **kwargs
+        kwargs['name'] = name
+        kwargs['flavor'] = flavor_id
+        kwargs['image'] = image_id or None
+        ephemeral = kwargs.pop('ephemeral', [])
+        block_device = kwargs.pop('block_device', [])
+        boot_volume = kwargs.pop('boot_volume', None)
+        snapshot = kwargs.pop('snapshot', None)
+        swap = kwargs.pop('swap', None)
+        kwargs['block_device_mapping_v2'] = _parse_block_device_mapping_v2(
+            block_device=block_device, boot_volume=boot_volume, snapshot=snapshot,
+            ephemeral=ephemeral, swap=swap
         )
+        response = nt_ks.servers.create(**kwargs)
         self.uuid = response.id
-        self.password = response.adminPass
+        self.password = getattr(response, 'adminPass', None)
 
         start = time.time()
         trycount = 0
@@ -295,6 +378,8 @@ class SaltNova(OpenStackComputeShell):
         '''
         Organize information about a volume from the volume_id
         '''
+        if self.volume_conn is None:
+            raise SaltCloudSystemExit('No cinder endpoint available')
         nt_ks = self.volume_conn
         volume = nt_ks.volumes.get(volume_id)
         response = {'name': volume.display_name,
@@ -310,6 +395,8 @@ class SaltNova(OpenStackComputeShell):
         '''
         List all block volumes
         '''
+        if self.volume_conn is None:
+            raise SaltCloudSystemExit('No cinder endpoint available')
         nt_ks = self.volume_conn
         volumes = nt_ks.volumes.list(search_opts=search_opts)
         response = {}
@@ -328,6 +415,8 @@ class SaltNova(OpenStackComputeShell):
         '''
         Show one volume
         '''
+        if self.volume_conn is None:
+            raise SaltCloudSystemExit('No cinder endpoint available')
         nt_ks = self.volume_conn
         volumes = self.volume_list(
             search_opts={'display_name': name},
@@ -345,6 +434,8 @@ class SaltNova(OpenStackComputeShell):
         '''
         Create a block device
         '''
+        if self.volume_conn is None:
+            raise SaltCloudSystemExit('No cinder endpoint available')
         nt_ks = self.volume_conn
         response = nt_ks.volumes.create(
             size=size,
@@ -360,6 +451,8 @@ class SaltNova(OpenStackComputeShell):
         '''
         Delete a block device
         '''
+        if self.volume_conn is None:
+            raise SaltCloudSystemExit('No cinder endpoint available')
         nt_ks = self.volume_conn
         try:
             volume = self.volume_show(name)
@@ -662,17 +755,20 @@ class SaltNova(OpenStackComputeShell):
         nt_ks = self.compute_conn
         ret = {}
         for item in nt_ks.servers.list():
-            ret[item.name] = {
-                'id': item.id,
-                'name': item.name,
-                'state': item.status,
-                'accessIPv4': item.accessIPv4,
-                'accessIPv6': item.accessIPv6,
-                'flavor': {'id': item.flavor['id'],
-                           'links': item.flavor['links']},
-                'image': {'id': item.image['id'],
-                          'links': item.image['links']},
-                }
+            try:
+                ret[item.name] = {
+                    'id': item.id,
+                    'name': item.name,
+                    'state': item.status,
+                    'accessIPv4': item.accessIPv4,
+                    'accessIPv6': item.accessIPv6,
+                    'flavor': {'id': item.flavor['id'],
+                               'links': item.flavor['links']},
+                    'image': {'id': item.image['id'] if item.image else 'Boot From Volume',
+                              'links': item.image['links'] if item.image else ''},
+                    }
+            except TypeError:
+                pass
         return ret
 
     def server_list_detailed(self):
@@ -682,28 +778,31 @@ class SaltNova(OpenStackComputeShell):
         nt_ks = self.compute_conn
         ret = {}
         for item in nt_ks.servers.list():
-            ret[item.name] = {
-                'OS-EXT-SRV-ATTR': {},
-                'OS-EXT-STS': {},
-                'accessIPv4': item.accessIPv4,
-                'accessIPv6': item.accessIPv6,
-                'addresses': item.addresses,
-                'created': item.created,
-                'flavor': {'id': item.flavor['id'],
-                           'links': item.flavor['links']},
-                'hostId': item.hostId,
-                'id': item.id,
-                'image': {'id': item.image['id'],
-                          'links': item.image['links']},
-                'key_name': item.key_name,
-                'links': item.links,
-                'metadata': item.metadata,
-                'name': item.name,
-                'state': item.status,
-                'tenant_id': item.tenant_id,
-                'updated': item.updated,
-                'user_id': item.user_id,
-            }
+            try:
+                ret[item.name] = {
+                    'OS-EXT-SRV-ATTR': {},
+                    'OS-EXT-STS': {},
+                    'accessIPv4': item.accessIPv4,
+                    'accessIPv6': item.accessIPv6,
+                    'addresses': item.addresses,
+                    'created': item.created,
+                    'flavor': {'id': item.flavor['id'],
+                               'links': item.flavor['links']},
+                    'hostId': item.hostId,
+                    'id': item.id,
+                    'image': {'id': item.image['id'] if item.image else 'Boot From Volume',
+                              'links': item.image['links'] if item.image else ''},
+                    'key_name': item.key_name,
+                    'links': item.links,
+                    'metadata': item.metadata,
+                    'name': item.name,
+                    'state': item.status,
+                    'tenant_id': item.tenant_id,
+                    'updated': item.updated,
+                    'user_id': item.user_id,
+                }
+            except TypeError:
+                continue
 
             ret[item.name]['progress'] = getattr(item, 'progress', '0')
 
@@ -741,9 +840,9 @@ class SaltNova(OpenStackComputeShell):
         ret = {}
         try:
             servers = self.server_list_detailed()
-        except AttributeError as exc:
+        except AttributeError:
             raise SaltCloudSystemExit('Corrupt server in server_list_detailed. Remove corrupt servers.')
-        for server_name, server in servers.iteritems():
+        for server_name, server in six.iteritems(servers):
             if str(server['id']) == server_id:
                 ret[server_name] = server
         return ret
@@ -828,7 +927,7 @@ class SaltNova(OpenStackComputeShell):
             'priority', 'project_id', 'vlan_start', 'vpn_start'
         ]
 
-        for variable in kwargs.keys():  # iterate over a copy, we might delete some
+        for variable in six.iterkeys(kwargs):  # iterate over a copy, we might delete some
             if variable not in params:
                 del kwargs[variable]
         return kwargs
@@ -869,94 +968,185 @@ class SaltNova(OpenStackComputeShell):
         nets = nt_ks.virtual_interfaces.create(networkid, serverid)
         return nets
 
-#The following is a list of functions that need to be incorporated in the
-#nova module. This list should be updated as functions are added.
+    def floating_ip_pool_list(self):
+        '''
+        List all floating IP pools
+
+        .. versionadded:: 2016.3.0
+        '''
+        nt_ks = self.compute_conn
+        pools = nt_ks.floating_ip_pools.list()
+        response = {}
+        for pool in pools:
+            response[pool.name] = {
+                'name': pool.name,
+            }
+        return response
+
+    def floating_ip_list(self):
+        '''
+        List floating IPs
+
+        .. versionadded:: 2016.3.0
+        '''
+        nt_ks = self.compute_conn
+        floating_ips = nt_ks.floating_ips.list()
+        response = {}
+        for floating_ip in floating_ips:
+            response[floating_ip.ip] = {
+                'ip': floating_ip.ip,
+                'fixed_ip': floating_ip.fixed_ip,
+                'id': floating_ip.id,
+                'instance_id': floating_ip.instance_id,
+                'pool': floating_ip.pool
+            }
+        return response
+
+    def floating_ip_show(self, ip):
+        '''
+        Show info on specific floating IP
+
+        .. versionadded:: 2016.3.0
+        '''
+        nt_ks = self.compute_conn
+        floating_ips = nt_ks.floating_ips.list()
+        for floating_ip in floating_ips:
+            if floating_ip.ip == ip:
+                return floating_ip
+        return {}
+
+    def floating_ip_create(self, pool=None):
+        '''
+        Allocate a floating IP
+
+        .. versionadded:: 2016.3.0
+        '''
+        nt_ks = self.compute_conn
+        floating_ip = nt_ks.floating_ips.create(pool)
+        response = {
+            'ip': floating_ip.ip,
+            'fixed_ip': floating_ip.fixed_ip,
+            'id': floating_ip.id,
+            'instance_id': floating_ip.instance_id,
+            'pool': floating_ip.pool
+        }
+        return response
+
+    def floating_ip_delete(self, floating_ip):
+        '''
+        De-allocate a floating IP
+
+        .. versionadded:: 2016.3.0
+        '''
+        ip = self.floating_ip_show(floating_ip)
+        nt_ks = self.compute_conn
+        return nt_ks.floating_ips.delete(ip)
+
+    def floating_ip_associate(self, server_name, floating_ip):
+        '''
+        Associate floating IP address to server
+
+        .. versionadded:: 2016.3.0
+        '''
+        nt_ks = self.compute_conn
+        server_ = self.server_by_name(server_name)
+        server = nt_ks.servers.get(server_.__dict__['id'])
+        server.add_floating_ip(floating_ip)
+        return self.floating_ip_list()[floating_ip]
+
+    def floating_ip_disassociate(self, server_name, floating_ip):
+        '''
+        Disassociate a floating IP from server
+
+        .. versionadded:: 2016.3.0
+        '''
+        nt_ks = self.compute_conn
+        server_ = self.server_by_name(server_name)
+        server = nt_ks.servers.get(server_.__dict__['id'])
+        server.remove_floating_ip(floating_ip)
+        return self.floating_ip_list()[floating_ip]
+
+# The following is a list of functions that need to be incorporated in the
+# nova module. This list should be updated as functions are added.
 #
-#absolute-limits     Print a list of absolute limits for a user
-#actions             Retrieve server actions.
-#add-fixed-ip        Add new IP address to network.
-#add-floating-ip     Add a floating IP address to a server.
-#aggregate-add-host  Add the host to the specified aggregate.
-#aggregate-create    Create a new aggregate with the specified details.
-#aggregate-delete    Delete the aggregate by its id.
-#aggregate-details   Show details of the specified aggregate.
-#aggregate-list      Print a list of all aggregates.
-#aggregate-remove-host
-#                    Remove the specified host from the specified aggregate.
-#aggregate-set-metadata
-#                    Update the metadata associated with the aggregate.
-#aggregate-update    Update the aggregate's name and optionally
-#                    availability zone.
-#cloudpipe-create    Create a cloudpipe instance for the given project
-#cloudpipe-list      Print a list of all cloudpipe instances.
-#console-log         Get console log output of a server.
-#credentials         Show user credentials returned from auth
-#describe-resource   Show details about a resource
-#diagnostics         Retrieve server diagnostics.
-#dns-create          Create a DNS entry for domain, name and ip.
-#dns-create-private-domain
-#                    Create the specified DNS domain.
-#dns-create-public-domain
-#                    Create the specified DNS domain.
-#dns-delete          Delete the specified DNS entry.
-#dns-delete-domain   Delete the specified DNS domain.
-#dns-domains         Print a list of available dns domains.
-#dns-list            List current DNS entries for domain and ip or domain
-#                    and name.
-#endpoints           Discover endpoints that get returned from the
-#                    authenticate services
-#floating-ip-create  Allocate a floating IP for the current tenant.
-#floating-ip-delete  De-allocate a floating IP.
-#floating-ip-list    List floating ips for this tenant.
-#floating-ip-pool-list
-#                    List all floating ip pools.
-#get-vnc-console     Get a vnc console to a server.
-#host-action         Perform a power action on a host.
-#host-update         Update host settings.
-#image-create        Create a new image by taking a snapshot of a running
-#                    server.
-#image-delete        Delete an image.
-#live-migration      Migrates a running instance to a new machine.
-#meta                Set or Delete metadata on a server.
-#migrate             Migrate a server.
-#pause               Pause a server.
-#rate-limits         Print a list of rate limits for a user
-#reboot              Reboot a server.
-#rebuild             Shutdown, re-image, and re-boot a server.
-#remove-fixed-ip     Remove an IP address from a server.
-#remove-floating-ip  Remove a floating IP address from a server.
-#rename              Rename a server.
-#rescue              Rescue a server.
-#resize              Resize a server.
-#resize-confirm      Confirm a previous resize.
-#resize-revert       Revert a previous resize (and return to the previous
-#                    VM).
-#root-password       Change the root password for a server.
-#secgroup-add-group-rule
-#                    Add a source group rule to a security group.
-#secgroup-add-rule   Add a rule to a security group.
-#secgroup-delete-group-rule
-#                    Delete a source group rule from a security group.
-#secgroup-delete-rule
-#                    Delete a rule from a security group.
-#secgroup-list-rules
-#                    List rules for a security group.
-#ssh                 SSH into a server.
-#unlock              Unlock a server.
-#unpause             Unpause a server.
-#unrescue            Unrescue a server.
-#usage-list          List usage data for all tenants
-#volume-list         List all the volumes.
-#volume-snapshot-create
-#                    Add a new snapshot.
-#volume-snapshot-delete
-#                    Remove a snapshot.
-#volume-snapshot-list
-#                    List all the snapshots.
-#volume-snapshot-show
-#                    Show details about a snapshot.
-#volume-type-create  Create a new volume type.
-#volume-type-delete  Delete a specific flavor
-#volume-type-list    Print a list of available 'volume types'.
-#x509-create-cert    Create x509 cert for a user in tenant
-#x509-get-root-cert  Fetches the x509 root cert.
+# absolute-limits     Print a list of absolute limits for a user
+# actions             Retrieve server actions.
+# add-fixed-ip        Add new IP address to network.
+# aggregate-add-host  Add the host to the specified aggregate.
+# aggregate-create    Create a new aggregate with the specified details.
+# aggregate-delete    Delete the aggregate by its id.
+# aggregate-details   Show details of the specified aggregate.
+# aggregate-list      Print a list of all aggregates.
+# aggregate-remove-host
+#                     Remove the specified host from the specified aggregate.
+# aggregate-set-metadata
+#                     Update the metadata associated with the aggregate.
+# aggregate-update    Update the aggregate's name and optionally
+#                     availability zone.
+# cloudpipe-create    Create a cloudpipe instance for the given project
+# cloudpipe-list      Print a list of all cloudpipe instances.
+# console-log         Get console log output of a server.
+# credentials         Show user credentials returned from auth
+# describe-resource   Show details about a resource
+# diagnostics         Retrieve server diagnostics.
+# dns-create          Create a DNS entry for domain, name and ip.
+# dns-create-private-domain
+#                     Create the specified DNS domain.
+# dns-create-public-domain
+#                     Create the specified DNS domain.
+# dns-delete          Delete the specified DNS entry.
+# dns-delete-domain   Delete the specified DNS domain.
+# dns-domains         Print a list of available dns domains.
+# dns-list            List current DNS entries for domain and ip or domain
+#                     and name.
+# endpoints           Discover endpoints that get returned from the
+#                     authenticate services
+# get-vnc-console     Get a vnc console to a server.
+# host-action         Perform a power action on a host.
+# host-update         Update host settings.
+# image-create        Create a new image by taking a snapshot of a running
+#                     server.
+# image-delete        Delete an image.
+# live-migration      Migrates a running instance to a new machine.
+# meta                Set or Delete metadata on a server.
+# migrate             Migrate a server.
+# pause               Pause a server.
+# rate-limits         Print a list of rate limits for a user
+# reboot              Reboot a server.
+# rebuild             Shutdown, re-image, and re-boot a server.
+# remove-fixed-ip     Remove an IP address from a server.
+# rename              Rename a server.
+# rescue              Rescue a server.
+# resize              Resize a server.
+# resize-confirm      Confirm a previous resize.
+# resize-revert       Revert a previous resize (and return to the previous
+#                     VM).
+# root-password       Change the root password for a server.
+# secgroup-add-group-rule
+#                     Add a source group rule to a security group.
+# secgroup-add-rule   Add a rule to a security group.
+# secgroup-delete-group-rule
+#                     Delete a source group rule from a security group.
+# secgroup-delete-rule
+#                     Delete a rule from a security group.
+# secgroup-list-rules
+#                     List rules for a security group.
+# ssh                 SSH into a server.
+# unlock              Unlock a server.
+# unpause             Unpause a server.
+# unrescue            Unrescue a server.
+# usage-list          List usage data for all tenants
+# volume-list         List all the volumes.
+# volume-snapshot-create
+#                     Add a new snapshot.
+# volume-snapshot-delete
+#                     Remove a snapshot.
+# volume-snapshot-list
+#                     List all the snapshots.
+# volume-snapshot-show
+#                     Show details about a snapshot.
+# volume-type-create  Create a new volume type.
+# volume-type-delete  Delete a specific flavor
+# volume-type-list    Print a list of available 'volume types'.
+# x509-create-cert    Create x509 cert for a user in tenant
+# x509-get-root-cert  Fetches the x509 root cert.
